{% load i18n static django_webix_utils %}

{% webix_version as webix_ver %}
{% webix_fontawesome_css_url as fontawesome_css_url %}
{% webix_fontawesome_version as fontawesome_version %}

<link rel="stylesheet" href="{% static 'webix/skins/compact.min.css' %}?ver={{ ver }}" type="text/css">
<link rel="stylesheet" href="{% static 'django_webix/inlines.css' %}" type="text/css">
<link rel="stylesheet" href="{% static 'django_webix/theme.siberia.css' %}" type="text/css">
<link rel="stylesheet" href="{% static fontawesome_css_url %}?ver={{ fontawesome_version }}" type="text/css">

<script type="text/javascript" src="{% static 'webix/webix.min.js' %}?ver={{ webix_ver }}"></script>
<script type="text/javascript" src="{% static 'django_webix/jquery-3.3.1.min.js' %}"></script>
<script type="text/javascript" charset="utf-8">
    {% include "django_webix/include/form_validation.js" %}
    {% include "django_webix/include/utils.js" %}

    webix.attachEvent("onBeforeAjax", function (mode, url, data, xhr, headers) {
        var key = document.querySelector("[name=csrfmiddlewaretoken]");
        if (key !== undefined && key !== null) {
            headers["X-CSRFToken"] = key.value;
        }
    });
</script>
